<?php

namespace SGalinski\SgNews\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgNews\Domain\Model\Author;
use SGalinski\SgNews\Domain\Model\Category;
use SGalinski\SgNews\Domain\Model\News;
use SGalinski\SgNews\Domain\Repository\AuthorRepository;
use SGalinski\SgNews\Domain\Repository\CategoryRepository;
use SGalinski\SgNews\Domain\Repository\NewsRepository;
use SGalinski\SgSeo\Service\HeadTagService;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * News Author List Controller
 */
class NewsByAuthorController extends AbstractController {
	/**
	 * Renders the news author list.
	 *
	 * @param int $authorId
	 * @return void
	 * @throws \InvalidArgumentException
	 * @throws \TYPO3\CMS\Extbase\Persistence\Exception\InvalidQueryException
	 * @throws \TYPO3\CMS\Core\Package\Exception
	 */
	public function listAction($authorId = 0): ?\Psr\Http\Message\ResponseInterface {
		if ($authorId) {
			$newsAuthorsIds[] = $authorId;
		} else {
			$newsAuthorsIds = GeneralUtility::intExplode(',', $this->settings['newsAuthors']);
			if (\count($newsAuthorsIds) <= 0) {
				if (version_compare(\TYPO3\CMS\Core\Utility\VersionNumberUtility::getCurrentTypo3Version(), '11.0.0', '<')) {
					return NULL;
				} else {
					return $this->htmlResponse();
				}
			}
		}

		$authors = [];
		$authorRepository = $this->objectManager->get(AuthorRepository::class);
		foreach ($newsAuthorsIds as $newsAuthorsId) {
			/** @var Author $author */
			$author = $authorRepository->findByUid($newsAuthorsId);
			if (!$author) {
				continue;
			}

			if (version_compare(ExtensionManagementUtility::getExtensionVersion('sg_seo'), '5.0.0', '>=')) {
				$headTagService = GeneralUtility::makeInstance(
					HeadTagService::class,
					TRUE,
					$author->getName(),
					$author->getDescription(),
					'&tx_sgnews_newsbyauthor[authorId]=' . $author->getUid()
				);
				$headTagService->execute();
			} else {
				$GLOBALS['TSFE']->page['titlebyextension'] = $author->getName();
				$GLOBALS['TSFE']->page['description'] = \strip_tags(\substr($author->getDescription(), 0, 200));
				$GLOBALS['TSFE']->page['extensionArgumentsForCanonicalAndHrefLang'] =
					'&tx_sgnews_newsbyauthor[authorId]=' . $author->getUid();
			}

			$authors[] = $author;
		}

		if (\count($authors) <= 0) {
			if (version_compare(\TYPO3\CMS\Core\Utility\VersionNumberUtility::getCurrentTypo3Version(), '11.0.0', '<')) {
				return NULL;
			} else {
				return $this->htmlResponse();
			}
		}

		$newsRepository = $this->objectManager->get(NewsRepository::class);
		$news = $newsRepository->findAllByNewsAuthor($newsAuthorsIds);
		if (\count($news) <= 0) {
			if (version_compare(\TYPO3\CMS\Core\Utility\VersionNumberUtility::getCurrentTypo3Version(), '11.0.0', '<')) {
				return NULL;
			} else {
				return $this->htmlResponse();
			}
		}

		$categories = $newsMetaData = [];
		$categoryRepository = $this->objectManager->get(CategoryRepository::class);
		$excludedNewsIds = GeneralUtility::intExplode(',', $this->settings['excludedNews']);
		foreach ($news as $newsResult) {
			foreach ($newsResult as $newsEntry) {
				/** @var News $newsEntry */
				if (in_array($newsEntry->getUid(), $excludedNewsIds, TRUE)) {
					continue;
				}

				$categoryId = $newsEntry->getPid();
				if (!isset($categories[$categoryId])) {
					/** @var Category $category */
					$category = $categoryRepository->findByUid($categoryId);
					if (!$category) {
						continue;
					}

					$categories[$categoryId] = $category;
				}

				$newsMetaData[] = $this->getMetaDataForNews($newsEntry, $categories[$categoryId]);
			}
		}

		$this->view->assign('newsMetaData', $newsMetaData);
		$this->view->assign('authors', $authors);

		if (version_compare(\TYPO3\CMS\Core\Utility\VersionNumberUtility::getCurrentTypo3Version(), '11.0.0', '<')) {
			return NULL;
		} else {
			return $this->htmlResponse();
		}
	}
}
